import java.io.*;
import java.util.*;
/** The class Simplex is the main program.<br>
 *  It is called with 3 parameter in the commandline.<br>
 *  First parameter is the input file with the tableau. <br>
 *  Second parameter is the output file with the result and<br>
 *  the third parameter is the logging file with step by step results.<br>
 *  It uses the class LinProg to call the multiphase and simplex method
 *  on the tableau that will be given with the input file.
 *  The input file must have the following construction:<br>
 *
<PRE>
n m
C_0   C_1   ...  C_N   C_N+1  ...   C_N+M C
A_11  A_12  ...  A_1N  1     0 ..   0     B_1

       ... ...              ...           ...

A_M1  A_M2  ...  A_MN  0      ..  0 1     B_M
</PRE>
 *  This tableau will be read from the input file and stored in a
 *  double-array. Then a permutation vector (short <b>pv</b>) will be created, which
 *  stores the "Basisvektoren". <br>
 *  The multiphase method will be called with the tableau and the
 *  pv. After that the simplex method will be called.
 *  If there was problem like an empty or unlimited "Loesungsraum" the
 *  program will report this in the logging and output file.
 *  Else the step by step results will be written to the logging file, and
 *  only the result to the output file.
 */
public class Simplex {


	public static void main(String[] args) {

		if (args.length<3) {
			System.err.println("Invalid option\n");
			System.err.println("Usage: java Simplex <input> <output> <logfile>\n");
			System.err.println("Programm aborted\n");
			System.exit(-1);
		}

		File input  = new File(args[0]);
		File output = new File(args[1]);
		File log    = new File(args[2]);

		// all logging information
		StringBuffer logging = new StringBuffer("Protocol\n\n");
		// the result
		StringBuffer result = new StringBuffer("Result: \n\n");


		BufferedReader infile  = null;
		BufferedWriter outfile = null;
		BufferedWriter logfile = null;

		Vector vec = new Vector();
		// trying to open input and output and logging
		try {
			infile  = new BufferedReader(new InputStreamReader (new FileInputStream(input)));
			outfile = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(output)));
			logfile = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(log)));
			// reading the input file and placing every line into an vector
			while (infile.ready()) {
				String text= "";
				while( (text = infile.readLine())!=null) {
					vec.add(text);
				}
			}
			infile.close();
		} catch (IOException exc) {
			exc.printStackTrace();
			System.err.println("Can't read input file.\nProgram aborted.\n");
			System.exit(-1);
		}

		int n = -1;
		int m = -1;

		// getting the n and m parameter out of the vector
		StringTokenizer tok = null;
		tok = new StringTokenizer((String)vec.get(0));
		n = Integer.parseInt(tok.nextToken());
		m = Integer.parseInt(tok.nextToken());

		// place given information of the dimension n and m
		// into the tableau
		double[][] tableau = new double[m+1][n+m+1];
		for (int row=0;row<=m;row++) {
			tok = new StringTokenizer((String)vec.get(row+1));
			for (int col=0;col<=n+m;col++) {
				tableau[row][col] = Double.parseDouble(tok.nextToken());
			}
		}

		// creating a permutation vector who stores the position of the "Basisvektoren"
		int[] pv;
		pv = LinProg.getPV(tableau);

		// start LP calling first multiphase and then simplex.
		// An empty or unlimited "Loesungsraum" will throw an Exception
		// and will be logged.
		logging.append(LinProg.getTableau(tableau)+"\n");
		try {
			logging.append(LinProg.multiphase(tableau,pv));
			logging.append(LinProg.simplex(tableau,pv));
			String res = LinProg.getResult(tableau,pv);
			logging.append(res);
			result.append(LinProg.getTableau(tableau)+"\n");
			result.append(res);
		} catch (UnsolvableException exc) {
			logging.append(exc.getMessage());
			result.append(exc.getMessage());
		}
		System.err.println(logging);

		try {
			outfile.write(result.toString());
			outfile.close();
		} catch (IOException exc) {
			exc.printStackTrace();
		}
		try {
			logfile.write(logging.toString());
			logfile.close();
		} catch (IOException exc) {
			exc.printStackTrace();
		}

	}
}